@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002 raf <raf@raf.org>
+* Copyright (C) 1999-2004 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20020916 raf <raf@raf.org>
+* 20040102 raf <raf@raf.org>
 */
 
 #ifndef LIBSLACK_HSORT_H
@@ -28,10 +28,12 @@
 
 #include <slack/hdr.h>
 
-typedef int hsort_cmp_t(const void *, const void *);
+typedef int hsort_cmp_t(const void *a, const void *b);
+typedef int hsort_closure_cmp_t(const void *a, const void *b, const void *data);
 
 _begin_decls
-void hsort _args ((void *base, size_t n, size_t size, hsort_cmp_t *cmp));
+void hsort(void *base, size_t n, size_t size, hsort_cmp_t *cmp);
+void hsort_closure(void *base, size_t n, size_t size, hsort_closure_cmp_t *cmp, const void *data);
 _end_decls
 
 #endif
