@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2001 raf <raf@raf.org>
+* Copyright (C) 1999-2004 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20011109 raf <raf@raf.org>
+* 20040102 raf <raf@raf.org>
 */
 
 /*
@@ -29,6 +29,7 @@ I<libslack(map)> - map module
 
 =head1 SYNOPSIS
 
+    #include <slack/std.h>
     #include <slack/map.h>
 
     typedef struct Map Map;
@@ -273,7 +274,7 @@ static void mapping_release(Mapping *mapping)
 
 Creates a small I<Map> with string keys and C<destroy> as its item
 destructor. It is the caller's responsibility to deallocate the new map with
-I<map_release()> or I<map_destroy()>. On success, returns the new map. On
+I<map_release(3)> or I<map_destroy(3)>. On success, returns the new map. On
 error, returns C<null> with C<errno> set appropriately.
 
 =cut
@@ -289,7 +290,7 @@ Map *map_create(map_release_t *destroy)
 
 =item C<Map *map_create_sized(size_t size, map_release_t *destroy)>
 
-Equivalent to I<map_create()> except that the initial number of buckets is
+Equivalent to I<map_create(3)> except that the initial number of buckets is
 approximately C<size>. The actual size will be the first prime greater than
 or equal to C<size> in a prebuilt sequence of primes between 11 and
 26,214,401 that double at each step.
@@ -307,7 +308,7 @@ Map *map_create_sized(size_t size, map_release_t *destroy)
 
 =item C<Map *map_create_with_hash(map_hash_t *hash, map_release_t *destroy)>
 
-Equivalent to  I<map_create()> except that C<hash> is used as the hash
+Equivalent to  I<map_create(3)> except that C<hash> is used as the hash
 function. The arguments to C<hash> are a I<size_t> specifying the number of
 buckets and a I<const void *> specifying the key to hash. It must return a
 I<size_t> between zero and the table size - 1.
@@ -325,7 +326,7 @@ Map *map_create_with_hash(map_hash_t *hash, map_release_t *destroy)
 
 =item C<Map *map_create_sized_with_hash(size_t size, map_hash_t *hash, map_release_t *destroy)>
 
-Equivalent to I<map_create_sized()> except that C<hash> is used as the hash
+Equivalent to I<map_create_sized(3)> except that C<hash> is used as the hash
 function. The arguments to C<hash> are a I<size_t> specifying the number of
 buckets and a I<const void *> specifying the key to hash. It must return a
 I<size_t> between zero and the table size - 1.
@@ -343,7 +344,7 @@ Map *map_create_sized_with_hash(size_t size, map_hash_t *hash, map_release_t *de
 
 =item C<Map *map_create_with_locker(Locker *locker, map_release_t *destroy)>
 
-Equivalent to I<map_create()> except that multiple threads accessing the
+Equivalent to I<map_create(3)> except that multiple threads accessing the
 new map will be synchronised by C<locker>.
 
 =cut
@@ -359,7 +360,7 @@ Map *map_create_with_locker(Locker *locker, map_release_t *destroy)
 
 =item C<Map *map_create_with_locker_sized(Locker *locker, size_t size, map_release_t *destroy)>
 
-Equivalent to I<map_create_sized()> except that multiple threads accessing
+Equivalent to I<map_create_sized(3)> except that multiple threads accessing
 the new map will be synchronised by C<locker>.
 
 =cut
@@ -375,8 +376,8 @@ Map *map_create_with_locker_sized(Locker *locker, size_t size, map_release_t *de
 
 =item C<Map *map_create_with_locker_with_hash(Locker *locker, map_hash_t *hash, map_release_t *destroy)>
 
-Equivalent to I<map_create_with_hash()> except that multiple threads accessing
-the new map will be synchronised by C<locker>.
+Equivalent to I<map_create_with_hash(3)> except that multiple threads
+accessing the new map will be synchronised by C<locker>.
 
 =cut
 
@@ -391,7 +392,7 @@ Map *map_create_with_locker_with_hash(Locker *locker, map_hash_t *hash, map_rele
 
 =item C<Map *map_create_with_locker_sized_with_hash(Locker *locker, size_t size, map_hash_t *hash, map_release_t *destroy)>
 
-Equivalent to I<map_create_sized_with_hash()> except that multiple threads
+Equivalent to I<map_create_sized_with_hash(3)> except that multiple threads
 accessing the new map will be synchronised by C<locker>.
 
 =cut
@@ -407,7 +408,7 @@ Map *map_create_with_locker_sized_with_hash(Locker *locker, size_t size, map_has
 
 =item C<Map *map_create_generic(map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_release_t *key_destroy, map_release_t *value_destroy)>
 
-Equivalent to I<map_create()> except that the mapping keys can be of any
+Equivalent to I<map_create(3)> except that the mapping keys can be of any
 type. C<copy> is used to copy mapping keys. The argument to C<copy> is the
 key to be copied. It must return a copy of its argument. C<cmp> is used to
 compare mapping keys. The arguments to C<cmp> are two keys to be compared.
@@ -433,7 +434,7 @@ Map *map_create_generic(map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_
 
 =item C<Map *map_create_generic_sized(size_t size, map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_release_t *key_destroy, map_release_t *value_destroy)>
 
-Equivalent to I<map_create_generic()> except that the initial number of
+Equivalent to I<map_create_generic(3)> except that the initial number of
 buckets is approximately C<size>. The actual size will be the first prime
 greater than or equal to C<size> in a prebuilt sequence of primes between 11
 and 26,214,401 that double at each step.
@@ -451,8 +452,8 @@ Map *map_create_generic_sized(size_t size, map_copy_t *copy, map_cmp_t *cmp, map
 
 =item C<Map *map_create_generic_with_locker(Locker *locker, map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_release_t *key_destroy, map_release_t *value_destroy)>
 
-Equivalent to I<map_create_generic()> except that multiple threads accessing
-the new map will be synchronised by C<locker>.
+Equivalent to I<map_create_generic(3)> except that multiple threads
+accessing the new map will be synchronised by C<locker>.
 
 =cut
 
@@ -467,7 +468,7 @@ Map *map_create_generic_with_locker(Locker *locker, map_copy_t *copy, map_cmp_t
 
 =item C<Map *map_create_generic_with_locker_sized(Locker *locker, size_t size, map_copy_t *copy, map_cmp_t *cmp, map_hash_t *hash, map_release_t *key_destroy, map_release_t *value_destroy)>
 
-Equivalent to I<map_create_generic_sized()> except that multiple threads
+Equivalent to I<map_create_generic_sized(3)> except that multiple threads
 accessing the new map will be synchronised by C<locker>.
 
 =cut
@@ -519,18 +520,17 @@ Map *map_create_generic_with_locker_sized(Locker *locker, size_t size, map_copy_
 =item C<int map_rdlock(const Map *map)>
 
 Claims a read lock on C<map> (if C<map> was created with a I<Locker>). This
-is needed when multiple read only I<map> module functions need to be called
-atomically. It is the client's responsibility to call I<map_unlock()> after
-the atomic operation. The only functions that may be called on C<map>
-between calls to I<map_rdlock()> and I<map_unlock()> are any read only map
-module functions whose name ends with C<_unlocked>. On success, returns
-C<0>. On error, returns an error code.
+is needed when multiple read only L<map(3)|map(3)> module functions need to
+be called atomically. It is the client's responsibility to call
+I<map_unlock(3)> after the atomic operation. The only functions that may be
+called on C<map> between calls to I<map_rdlock(3)> and I<map_unlock(3)> are
+any read only L<map(3)|map(3)> module functions whose name ends with
+C<_unlocked>. On success, returns C<0>. On error, returns an error code.
 
 =cut
 
 */
 
-
 #define map_rdlock(map) ((map) ? locker_rdlock((map)->locker) : EINVAL)
 #define map_wrlock(map) ((map) ? locker_wrlock((map)->locker) : EINVAL)
 #define map_unlock(map) ((map) ? locker_unlock((map)->locker) : EINVAL)
@@ -545,12 +545,12 @@ int (map_rdlock)(const Map *map)
 =item C<int map_wrlock(const Map *map)>
 
 Claims a write lock on C<map> (if C<map> was created with a I<Locker>). This
-is needed when multiple read/write I<map> module functions need to be called
-atomically. It is the client's responsibility to subsequently call
-I<map_unlock()>. The only functions that may be called on C<map> between
-calls to I<map_wrlock()> and I<map_unlock()> are any map module functions
-whose name ends with C<_unlocked>. On success, returns C<0>. On error,
-returns an error code.
+is needed when multiple read/write L<map(3)|map(3)> module functions need to
+be called atomically. It is the client's responsibility to subsequently call
+I<map_unlock(3)>. The only functions that may be called on C<map> between
+calls to I<map_wrlock(3)> and I<map_unlock(3)> are any L<map(3)|map(3)>
+module functions whose name ends with C<_unlocked>. On success, returns
+C<0>. On error, returns an error code.
 
 =cut
 
@@ -565,8 +565,8 @@ int (map_wrlock)(const Map *map)
 
 =item C<int map_unlock(const Map *map)>
 
-Unlocks a read or write lock on C<map> obtained with I<map_rdlock()> or
-I<map_wrlock()> (if C<map> was created with a C<locker>). On success,
+Unlocks a read or write lock on C<map> obtained with I<map_rdlock(3)> or
+I<map_wrlock(3)> (if C<map> was created with a C<locker>). On success,
 returns C<0>. On error, returns an error code.
 
 =cut
@@ -662,7 +662,7 @@ int map_own(Map *map, map_release_t *destroy)
 
 =item C<int map_own_unlocked(Map *map, map_release_t *destroy)>
 
-Equivalent to I<map_own()> except that C<map> is not write locked.
+Equivalent to I<map_own(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -737,7 +737,7 @@ map_release_t *map_disown(Map *map)
 
 =item C<map_release_t *map_disown_unlocked(Map *map)>
 
-Equivalent to I<map_disown()> except that C<map> is not write locked.
+Equivalent to I<map_disown(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -847,6 +847,7 @@ static int map_resize(Map *map)
 	map->size = new_map->size;
 	map->items = new_map->items;
 	map->chain = new_map->chain;
+	map->value_destroy = new_map->value_destroy;
 	mem_release(new_map);
 
 	return 0;
@@ -873,7 +874,7 @@ int map_add(Map *map, const void *key, void *value)
 
 =item C<int map_add_unlocked(Map *map, const void *key, void *value)>
 
-Equivalent to I<map_add()> except that C<map> is not write locked.
+Equivalent to I<map_add(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -905,7 +906,7 @@ int map_put(Map *map, const void *key, void *value)
 
 =item C<int map_put_unlocked(Map *map, const void *key, void *value)>
 
-Equivalent to I<map_put()> except that C<map> is not write locked.
+Equivalent to I<map_put(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -953,7 +954,7 @@ int map_insert(Map *map, const void *key, void *value, int replace)
 
 =item C<int map_insert_unlocked(Map *map, const void *key, void *value, int replace)>
 
-Equivalent to I<map_insert()> except that C<map> is not write locked.
+Equivalent to I<map_insert(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -1047,7 +1048,7 @@ int map_remove(Map *map, const void *key)
 
 =item C<int map_remove_unlocked(Map *map, const void *key)>
 
-Equivalent to I<map_remove()> except that C<map> is not write locked.
+Equivalent to I<map_remove(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -1123,7 +1124,7 @@ void *map_get(Map *map, const void *key)
 
 =item C<void *map_get_unlocked(const Map *map, const void *key)>
 
-Equivalent to I<map_get()> except that C<map> is not read locked.
+Equivalent to I<map_get(3)> except that C<map> is not read locked.
 
 =cut
 
@@ -1163,7 +1164,7 @@ void *map_get_unlocked(const Map *map, const void *key)
 =item C<Mapper *mapper_create(Map *map)>
 
 Creates an iterator for C<map>. The iterator keeps C<map> write locked until
-it is released with I<mapper_release()> or I<mapper_destroy()>. Note that
+it is released with I<mapper_release(3)> or I<mapper_destroy(3)>. Note that
 the iterator itself is not locked so it must not be shared between threads.
 On success, returns the iterator. On error, returns C<null> with C<errno>
 set appropriately.
@@ -1181,9 +1182,9 @@ Mapper *mapper_create(Map *map)
 
 =item C<Mapper *mapper_create_rdlocked(Map *map)>
 
-Equivalent to I<mapper_create()> except that C<map> is read locked rather
-than write locked. Use this in preference to I<mapper_create()> when no
-calls to I<mapper_remove()> will be made during the iteration.
+Equivalent to I<mapper_create(3)> except that C<map> is read locked rather
+than write locked. Use this in preference to I<mapper_create(3)> when no
+calls to I<mapper_remove(3)> will be made during the iteration.
 
 =cut
 
@@ -1206,7 +1207,7 @@ Mapper *mapper_create_rdlocked(Map *map)
 
 =item C<Mapper *mapper_create_wrlocked(Map *map)>
 
-Equivalent to I<mapper_create()> except that this function name makes the
+Equivalent to I<mapper_create(3)> except that this function name makes the
 fact that C<map> is write locked explicit.
 
 =cut
@@ -1230,7 +1231,7 @@ Mapper *mapper_create_wrlocked(Map *map)
 
 =item C<Mapper *mapper_create_unlocked(Map *map)>
 
-Equivalent to I<mapper_create()> except that C<map> is not write locked.
+Equivalent to I<mapper_create(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -1285,7 +1286,7 @@ void mapper_release(Mapper *mapper)
 
 =item C<void mapper_release_unlocked(Mapper *mapper)>
 
-Equivalent to I<mapper_release()> except that the associated map is not
+Equivalent to I<mapper_release(3)> except that the associated map is not
 unlocked.
 
 =cut
@@ -1326,7 +1327,7 @@ void *mapper_destroy(Mapper **mapper)
 
 =item C<void *mapper_destroy_unlocked(Mapper **mapper)>
 
-Equivalent to I<mapper_destroy()> except that the associated map is not
+Equivalent to I<mapper_destroy(3)> except that the associated map is not
 unlocked.
 
 =cut
@@ -1456,7 +1457,7 @@ const Mapping *mapper_next_mapping(Mapper *mapper)
 
 Removes the current item in the iteration C<mapper>. The next item in the
 iteration is the item following the removed item, if any. This must be
-called after I<mapper_next()>. On error, sets C<errno> appropriately.
+called after I<mapper_next(3)>. On error, sets C<errno> appropriately.
 
 =cut
 
@@ -1488,17 +1489,17 @@ Returns whether or not there is another item in C<map> using an internal
 iterator. The first time this is called, a new internal I<Mapper> will be
 created (Note: There can be only one). When there are no more items, returns
 C<0> and destroys the internal iterator. When it returns C<1>, use
-I<map_next()> to retrieve the next item. On error, returns C<-1> with
+I<map_next(3)> to retrieve the next item. On error, returns C<-1> with
 C<errno> set appropriately.
 
 Note: If an iteration using an internal iterator terminates before the end
-of the map, it is the caller's responsibility to call I<map_break()>.
+of the map, it is the caller's responsibility to call I<map_break(3)>.
 Failure to do so will cause the internal iterator to leak. It will also
-break the next call to I<map_has_next()> which will continue where the
+break the next call to I<map_has_next(3)> which will continue where the
 current iteration stopped rather than starting at the beginning again.
-I<map_release()> assumes that there is no internal iterator so it is the
-caller's responsibility to complete the iteration or call I<map_break()>
-before releasing C<map> with I<map_release()> or I<map_destroy()>.
+I<map_release(3)> assumes that there is no internal iterator so it is the
+caller's responsibility to complete the iteration or call I<map_break(3)>
+before releasing C<map> with I<map_release(3)> or I<map_destroy(3)>.
 
 Note: The internal I<Mapper> does not lock C<map> so this function is not
 threadsafe. It can only be used with maps created in the current function
@@ -1507,10 +1508,10 @@ observed even in single threaded applications to avoid breaking iterator
 semantics (possible with nested function calls). If C<map> is a parameter or
 a variable declared outside the function, it is best to create an explicit
 I<Mapper> instead. If this function is used on such maps instead, it is the
-caller's responsibility to explicitly lock C<map> first with I<map_wrlock()>
-and explicitly unlock it with I<map_unlock()>. Do this even if you are
-writing single threaded code in case your function may one day be used in a
-multi threaded application.
+caller's responsibility to explicitly lock C<map> first with
+I<map_wrlock(3)> and explicitly unlock it with I<map_unlock(3)>. Do this
+even if you are writing single threaded code in case your function may one
+day be used in a multi threaded application.
 
 =cut
 
@@ -1536,9 +1537,9 @@ int map_has_next(Map *map)
 
 =item C<void map_break(Map *map)>
 
-Unlocks C<map> and destroys its internal iterator. Must be used only when
-an iteration using an internal iterator has terminated before reaching the
-end of C<map>. On error, returns C<null> with C<errno> set appropriately.
+Unlocks C<map> and destroys its internal iterator. Must be used only when an
+iteration using an internal iterator has terminated before reaching the end
+of C<map>. On error, returns C<null> with C<errno> set appropriately.
 
 =cut
 
@@ -1599,7 +1600,7 @@ const Mapping *map_next_mapping(Map *map)
 
 Removes the current item in C<map> using it's internal iterator. The next
 item in the iteration is the item following the removed item, if any. This
-must be called after I<map_next()>.
+must be called after I<map_next(3)>.
 
 =cut
 
@@ -1659,10 +1660,10 @@ const void *mapping_value(const Mapping *mapping)
 =item C<List *map_keys(Map *map)>
 
 Creates and returns a list of all of the keys contained in C<map>. It is the
-caller's responsibility to deallocate the new list with I<list_release()> or
-I<list_destroy()>. The keys in the new list are owned by C<map> so the list
-returned must not outlive C<map>. On error, returns C<null> with C<errno>
-set appropriately.
+caller's responsibility to deallocate the new list with I<list_release(3)>
+or I<list_destroy(3)>. The keys in the new list are owned by C<map> so the
+list returned must not outlive C<map>. On error, returns C<null> with
+C<errno> set appropriately.
 
 =cut
 
@@ -1677,7 +1678,7 @@ List *map_keys(Map *map)
 
 =item C<List *map_keys_unlocked(Map *map)>
 
-Equivalent to I<map_keys()> except that C<map> is not read locked.
+Equivalent to I<map_keys(3)> except that C<map> is not read locked.
 
 =cut
 
@@ -1692,7 +1693,7 @@ List *map_keys_unlocked(Map *map)
 
 =item C<List *map_keys_with_locker(Locker *locker, Map *map)>
 
-Equivalent to I<map_keys()> except that multiple threads accessing the list
+Equivalent to I<map_keys(3)> except that multiple threads accessing the list
 returned will be synchronised by C<locker>.
 
 =cut
@@ -1722,7 +1723,8 @@ List *map_keys_with_locker(Locker *locker, Map *map)
 
 =item C<List *map_keys_with_locker_unlocked(Locker *locker, Map *map)>
 
-Equivalent to I<map_keys_with_locker()> except that C<map> is not read locked.
+Equivalent to I<map_keys_with_locker(3)> except that C<map> is not read
+locked.
 
 =cut
 
@@ -1745,7 +1747,7 @@ List *map_keys_with_locker_unlocked(Locker *locker, Map *map)
 		return NULL;
 	}
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 
@@ -1767,9 +1769,9 @@ List *map_keys_with_locker_unlocked(Locker *locker, Map *map)
 
 Creates and returns a list of all of the values contained in C<map>. It is
 the caller's responsibility to deallocate the new list with
-I<list_release()> or I<list_destroy()>. The values in the list are not owned
-by the list so it must not outlive the owner of the items in C<map>. On
-error, returns C<null> with C<errno> set appropriately.
+I<list_release(3)> or I<list_destroy(3)>. The values in the list are not
+owned by the list so it must not outlive the owner of the items in C<map>.
+On error, returns C<null> with C<errno> set appropriately.
 
 =cut
 
@@ -1784,7 +1786,7 @@ List *map_values(Map *map)
 
 =item C<List *map_values_unlocked(Map *map)>
 
-Equivalent to I<map_values()> except that C<map> is not read locked.
+Equivalent to I<map_values(3)> except that C<map> is not read locked.
 
 =cut
 
@@ -1799,8 +1801,8 @@ List *map_values_unlocked(Map *map)
 
 =item C<List *map_values_with_locker(Locker *locker, Map *map)>
 
-Equivalent to I<map_values()> except that multiple threads accessing the list
-returned will be synchronised by C<locker>.
+Equivalent to I<map_values(3)> except that multiple threads accessing the
+list returned will be synchronised by C<locker>.
 
 =cut
 
@@ -1829,7 +1831,8 @@ List *map_values_with_locker(Locker *locker, Map *map)
 
 =item C<List *map_values_with_locker_unlocked(Locker *locker, Map *map)>
 
-Equivalent to I<map_values_with_locker()> except that C<map> is not read locked.
+Equivalent to I<map_values_with_locker(3)> except that C<map> is not read
+locked.
 
 =cut
 
@@ -1852,7 +1855,7 @@ List *map_values_with_locker_unlocked(Locker *locker, Map *map)
 		return NULL;
 	}
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 
@@ -1889,8 +1892,8 @@ void map_apply(Map *map, map_action_t *action, void *data)
 
 =item C<void map_apply_rdlocked(Map *map, map_action_t *action, void *data)>
 
-Equivalent to I<map_apply()> except that C<map> is read locked rather than
-write locked. Use this in preference to I<map_apply()> when C<map> and its
+Equivalent to I<map_apply(3)> except that C<map> is read locked rather than
+write locked. Use this in preference to I<map_apply(3)> when C<map> and its
 items will not be modified during the iteration.
 
 =cut
@@ -1926,8 +1929,8 @@ void map_apply_rdlocked(Map *map, map_action_t *action, void *data)
 
 =item C<void map_apply_wrlocked(Map *map, map_action_t *action, void *data)>
 
-Equivalent to I<map_apply()> except that this function name makes the
-fact that C<map> is write locked explicit.
+Equivalent to I<map_apply(3)> except that this function name makes the fact
+that C<map> is write locked explicit.
 
 =cut
 
@@ -1962,7 +1965,7 @@ void map_apply_wrlocked(Map *map, map_action_t *action, void *data)
 
 =item C<void map_apply_unlocked(Map *map, map_action_t *action, void *data)>
 
-Equivalent to I<map_apply()> except that C<map> is not write locked.
+Equivalent to I<map_apply(3)> except that C<map> is not write locked.
 
 =cut
 
@@ -1981,7 +1984,7 @@ void map_apply_unlocked(Map *map, map_action_t *action, void *data)
 	if (!(mapper = mapper_create_unlocked(map)))
 		return;
 
-	while (mapper_has_next(mapper))
+	while (mapper_has_next(mapper) == 1)
 	{
 		const Mapping *mapping = mapper_next_mapping(mapper);
 		action(mapping->key, mapping->value, data);
@@ -2024,7 +2027,7 @@ ssize_t map_size(Map *map)
 
 =item C<ssize_t map_size_unlocked(const Map *map)>
 
-Equivalent to I<map_size()> except that C<map> is not read locked.
+Equivalent to I<map_size(3)> except that C<map> is not read locked.
 
 =cut
 
@@ -2054,7 +2057,7 @@ When arguments are C<null> or out of range.
 
 =item ENOENT
 
-When I<map_get()> tries to get or I<map_remove()> tries to remove a
+When I<map_get(3)> tries to get or I<map_remove(3)> tries to remove a
 non-existent mapping.
 
 =back
@@ -2067,20 +2070,21 @@ By default, I<Map>s are not MT-Safe because most programs are single
 threaded and synchronisation doesn't come for free. Even in multi threaded
 programs, not all I<Map>s are necessarily shared between multiple threads.
 
-When a I<Map> is shared between multiple threads which need to be synchronised,
-the method of synchronisation must be carefully selected by the client code.
-There are tradeoffs between concurrency and overhead. The greater the
-concurrency, the greater the overhead. More locks give greater concurrency but
-have greater overhead. Readers/Writer locks can give greater concurrency than
-Mutex locks but have greater overhead. One lock for each I<Map> may be
-required, or one lock for all (or a set of) I<Map>s may be more appropriate.
+When a I<Map> is shared between multiple threads which need to be
+synchronised, the method of synchronisation must be carefully selected by
+the client code. There are tradeoffs between concurrency and overhead. The
+greater the concurrency, the greater the overhead. More locks give greater
+concurrency but have greater overhead. Readers/Writer locks can give greater
+concurrency than Mutex locks but have greater overhead. One lock for each
+I<Map> may be required, or one lock for all (or a set of) I<Map>s may be
+more appropriate.
 
-Generally, the best synchronisation strategy for a given application can only
-be determined by testing/benchmarking the written application. It is important
-to be able to experiment with the synchronisation strategy at this stage of
-development without pain.
+Generally, the best synchronisation strategy for a given application can
+only be determined by testing/benchmarking the written application. It is
+important to be able to experiment with the synchronisation strategy at this
+stage of development without pain.
 
-To facilitate this, I<Map>s can be created with I<map_create_with_locker()>
+To facilitate this, I<Map>s can be created with I<map_create_with_locker(3)>
 which takes a I<Locker> argument. The I<Locker> specifies a lock and a set
 of functions for manipulating the lock. Each I<Map> can have it's own lock
 by creating a separate I<Locker> for each I<Map>. Multiple I<Map>s can share
@@ -2091,12 +2095,147 @@ basis.
 I<MT-Disciplined> means that the application developer has a mechanism for
 specifying the synchronisation requirements to be applied to library code.
 
-=head1 BUGS
+=head1 EXAMPLES
 
-A C<null> pointer can't be a key. Neither can zero when using integers as keys.
+Create a map that doesn't own its items, populate it and then iterate over
+its values with the internal iterator to print the values:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    int main()
+    {
+        Map *map;
+
+        if (!(map = map_create(NULL)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", "123");
+        map_add(map, "def", "456");
+        map_add(map, "ghi", "789");
+
+        while (map_has_next(map) == 1)
+            printf("%s\n", map_next(map));
+
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+Create a map that does own its items, populate it and then iterate over
+its items with an external iterator to print its keys and values:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    int main()
+    {
+        Map *map;
+        Mapper *mapper;
+
+        if (!(map = map_create(free)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", strdup("123"));
+        map_add(map, "def", strdup("456"));
+        map_add(map, "ghi", strdup("789"));
+
+        if (!(mapper = mapper_create(map)))
+        {
+            map_destroy(&map);
+            return EXIT_FAILURE;
+        }
+
+        while (mapper_has_next(mapper) == 1)
+        {
+            const Mapping *mapping = mapper_next_mapping(mapper);
+
+            printf("%s -> %s\n", mapping_key(mapping), mapping_value(mapping));
+        }
+
+        mapper_destroy(&mapper);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+The same but with an apply function:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    void action(void *key, void *item, void *data)
+    {
+        printf("%s -> %s\n", (char *)key, (char *)item);
+    }
+
+    int main()
+    {
+        Map *map;
+
+        if (!(map = map_create(free)))
+            return EXIT_FAILURE;
+
+        map_add(map, "abc", strdup("123"));
+        map_add(map, "def", strdup("456"));
+        map_add(map, "ghi", strdup("789"));
+        map_apply(map, action, NULL);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+The same but with integer keys:
+
+    #include <slack/std.h>
+    #include <slack/map.h>
+
+    static void *int_copy(const void *key)
+    {
+        return (void *)key;
+    }
+
+    static int int_cmp(const void *a, const void *b)
+    {
+        return (int)a - (int)b;
+    }
+
+    static size_t int_hash(size_t size, const void *key)
+    {
+        return (int)key % size;
+    }
+
+    void action(void *key, void *item, void *data)
+    {
+        printf("%d -> %s\n", (int)key, (char *)item);
+    }
+
+    int main(int ac, char **av)
+    {
+        Map *map;
+
+        if (!(map = map_create_generic(int_copy, int_cmp, int_hash, NULL, free)))
+            return EXIT_FAILURE;
+
+        map_add(map, (void *)123, strdup("abc"));
+        map_add(map, (void *)456, strdup("def"));
+        map_add(map, (void *)789, strdup("ghi"));
+
+        map_apply(map, action, NULL);
+        map_destroy(&map);
+
+        return EXIT_SUCCESS;
+    }
+
+=head1 CAVEAT
+
+A C<null> pointer can't be a key. Neither can zero when using integers as
+keys.
 
 If you use an internal iterator in a loop that terminates before the end of
-the map, and fail to call I<map_break()>, the internal iterator will leak.
+the map, and fail to call I<map_break(3)>, the internal iterator will leak.
+
+=head1 BUGS
 
 Uses I<malloc(3)>. The type of memory used and the allocation strategy need
 to be decoupled from this code.
@@ -2110,7 +2249,7 @@ L<locker(3)|locker(3)>
 
 =head1 AUTHOR
 
-20011109 raf <raf@raf.org>
+20040102 raf <raf@raf.org>
 
 =cut
 
@@ -2184,7 +2323,7 @@ static void map_histogram(const char *name, Map *map)
 		size_t length = list_length(map->chain[i]);
 		if (length == -1)
 		{
-			printf("  length[%d] = -1\n", i);
+			printf("  length[%d] = -1\n", (int)i);
 			continue;
 		}
 		++histogram[length];
@@ -2194,7 +2333,7 @@ static void map_histogram(const char *name, Map *map)
 
 	for (i = 0; i < map->items; ++i)
 		if (histogram[i])
-			printf("    %d chain%s of length %d\n", histogram[i], (histogram[i] == 1) ? "" : "s", i);
+			printf("    %d chain%s of length %d\n", histogram[i], (histogram[i] == 1) ? "" : "s", (int)i);
 
 	printf("}\n");
 }
@@ -2232,7 +2371,7 @@ static void test_hash(void)
 
 	fclose(words);
 
-	printf("%d entries into %d buckets:\n\n", map->items, map->size);
+	printf("%d entries into %d buckets:\n\n", (int)map->items, (int)map->size);
 
 	for (c = 0; c < map->size; ++c)
 	{
@@ -2243,7 +2382,7 @@ static void test_hash(void)
 
 		if ((length = list_length(map->chain[c])) == -1)
 		{
-			printf(" length[%d] == -1\n", c);
+			printf(" length[%d] == -1\n", (int)c);
 			continue;
 		}
 
@@ -2255,8 +2394,8 @@ static void test_hash(void)
 	}
 
 	printf("avg = %g\n", (double)sum / (double)map->size);
-	printf("min = %d\n", min);
-	printf("max = %d\n", max);
+	printf("min = %d\n", (int)min);
+	printf("max = %d\n", (int)max);
 	map_histogram("dict", map);
 	map_release(map);
 
@@ -2340,7 +2479,11 @@ static size_t direct_hash(size_t size, int key)
 Map *mtmap = NULL;
 Locker *locker = NULL;
 pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+#ifdef PTHREAD_RWLOCK_INITIALIZER
 pthread_rwlock_t rwlock = PTHREAD_RWLOCK_INITIALIZER;
+#else
+pthread_rwlock_t rwlock;
+#endif
 int barrier[2];
 int size[2];
 const int lim = 1000;
@@ -2411,7 +2554,7 @@ void *iterate_builtin(void *arg)
 	{
 		map_wrlock(mtmap);
 
-		while (map_has_next(mtmap))
+		while (map_has_next(mtmap) == 1)
 		{
 			int val = (int)map_next(mtmap);
 
@@ -2636,7 +2779,7 @@ int main(int ac, char **av)
 	if (ac == 2 && !strcmp(av[1], "hash"))
 		test_hash();
 
-	printf("Testing: map\n");
+	printf("Testing: %s\n", "map");
 
 	/* Test map_create, map_add, map_get */
 
@@ -2714,7 +2857,7 @@ int main(int ac, char **av)
 
 		TEST_ACT(21, keys = list_create(NULL))
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			void *item;
 
@@ -2730,10 +2873,10 @@ int main(int ac, char **av)
 		TEST_ACT(24, keys = list_create(NULL))
 		TEST_ACT(24, values = list_create(NULL))
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			const Mapping *mapping;
-			
+
 			TEST_ACT(25, mapping = map_next_mapping(map))
 			else
 			{
@@ -2830,7 +2973,7 @@ int main(int ac, char **av)
 			while (mapper_has_next(mapper) == 1)
 			{
 				void *item;
-				
+
 				TEST_ACT(64, item = mapper_next(mapper))
 				mapper_remove(mapper);
 			}
@@ -2857,10 +3000,10 @@ int main(int ac, char **av)
 		TEST_ACT(71, map_add(map, "3", "3") == 0)
 		TEST_ACT(72, map_add(map, "4", "4") == 0)
 
-		while (map_has_next(map))
+		while (map_has_next(map) == 1)
 		{
 			void *item;
-			
+
 			TEST_ACT(73, item = map_next(map))
 			map_remove_current(map);
 		}
@@ -2991,6 +3134,10 @@ int main(int ac, char **av)
 	if (debug)
 		setbuf(stdout, NULL);
 
+#ifndef PTHREAD_RWLOCK_INITIALIZER
+	pthread_rwlock_init(&rwlock, NULL);
+#endif
+
 	if (debug)
 		locker = locker_create_debug_rwlock(&rwlock);
 	else
@@ -3020,7 +3167,7 @@ int main(int ac, char **av)
 	/* Test assumption: sizeof(int) <= sizeof(void *) */
 
 	if (sizeof(int) > sizeof(void *))
-		++errors, printf("Test 222: assumption failed: sizeof(int) > sizeof(void *): int maps are limited to %d bytes\n", sizeof(void *));
+		++errors, printf("Test 222: assumption failed: sizeof(int) > sizeof(void *): int maps are limited to %d bytes\n", (int)sizeof(void *));
 
 	/* Test assumption: memset(&ptr, 0, sizeof(void *)) same as NULL */
 
