@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2001 raf <raf@raf.org>
+* Copyright (C) 1999-2004 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20010215 raf <raf@raf.org>
+* 20040102 raf <raf@raf.org>
 */
 
 /*
@@ -29,6 +29,7 @@ I<libslack(prop)> - program properties file module
 
 =head1 SYNOPSIS
 
+    #include <slack/std.h>
     #include <slack/prop.h>
 
     const char *prop_get(const char *name);
@@ -45,14 +46,15 @@ I<libslack(prop)> - program properties file module
     int prop_set_bool(const char *name, int value);
     int prop_unset(const char *name);
     int prop_save(void);
-
+    int prop_clear(void);
+    int prop_locker(Locker *locker);
 
 =head1 DESCRIPTION
 
 This module provides support for system-wide and user-specific (generic and
 program-specific) properties in "well-known" locations:
 
-    /etc/properties.app             - system-wide, generic properties
+    /etc/properties/app             - system-wide, generic properties
     $HOME/.properties/app           - user-defined, generic properties
     /etc/properties/app.progname    - system-wide, program-specific properties
     $HOME/.properties/app.progname  - user-defined, program-specific properties
@@ -65,13 +67,20 @@ properties. The client can change properties at runtime and save the current
 properties back to disk (to the user-defined, program-specific properties
 file).
 
-Program names (as returned by I<prog_name()> are converted into file name
-suffixes by replacing every occurrance of the file path separator (``/'')
-with a ``-''. Properties files consist of one property per line. Each
-property is specified by its name, followed by C<``=''> followed by its
-value. The name must not have a C<``=''> in it unless it is quoted with a
-C<``\''>. The properties files may also contain blank lines and comments
-(C<``#''> until the end of the line).
+Program names (as returned by I<prog_name(3)>) are converted into file name
+suffixes by replacing every occurrence of the file path separator (C<'/'>)
+with a C<'-'>. Properties files consist of one property per line.
+
+Each property is specified by its name, followed by C<'='> followed by its
+value. The name must not have a C<'='> in it unless it is quoted with a
+preceding C<'\'>. Special characters are expressed in C string literal
+notation (e.g. C<\a\b\f\n\r\t\v\x1b>). Spaces immediately before and after
+the C<'='> are stripped unless they are quoted with a preceding C<'\'>. The
+properties files may also contain blank lines and comments (C<'#'> until the
+end of the line).
+
+Boolean property values can be expressed as C<0> or C<1>, C<true> or
+C<false>, C<yes> or C<no>, C<on> or C<off> (case insensitive).
 
 =over 4
 
@@ -79,6 +88,7 @@ C<``\''>. The properties files may also contain blank lines and comments
 
 */
 
+#include "config.h"
 #include "std.h"
 
 #include <pwd.h>
@@ -92,14 +102,18 @@ C<``\''>. The properties files may also contain blank lines and comments
 #include "mem.h"
 #include "prop.h"
 #include "str.h"
-#include "thread.h"
+#include "locker.h"
 
-#ifdef NEEDS_SNPRINTF
+#ifndef HAVE_SNPRINTF
 #include "snprintf.h"
 #endif
 
 #ifndef isodigit
-#define isodigit(c) (isdigit(c) && (c) < '8')
+#define isodigit(c) (isdigit((int)(unsigned char)c) && (unsigned char)(c) < '8')
+#endif
+
+#ifndef is_space
+#define is_space(c) isspace((int)(unsigned char)(c))
 #endif
 
 typedef struct Prop Prop;
@@ -110,6 +124,8 @@ struct Prop
 	Prop *defaults;
 };
 
+#ifndef TEST
+
 static struct
 {
 	int init;
@@ -125,8 +141,8 @@ g = { 0, NULL, NULL, 0, NULL };
 C<Prop *prop_create(Map *map, Prop *defaults)>
 
 Creates and returns a I<Prop> containing C<map> and C<defaults>. On error,
-returns C<NULL>. The new I<Prop> will destroy C<map> and C<defaults> when it
-is destroyed.
+returns C<null> with C<errno> set appropriately. The new I<Prop> will
+destroy C<map> and C<defaults> when it is destroyed.
 
 */
 
@@ -182,7 +198,7 @@ static const char *eq = "=";
 
 C<String *quote_special(const char *src)>
 
-Replaces every occurrance in C<src> of special characters (represented in
+Replaces every occurrence in C<src> of special characters (represented in
 I<C> by C<"\a">, C<"\b">, C<"\f">, C<"\n">, C<"\r">, C<"\t">, C<"\v">) with
 their corresponding I<C> representation. Other non-printable characters are
 replaced with their ASCII codes in hexadecimal (i.e. "\xhh"). The caller
@@ -199,7 +215,7 @@ static String *quote_special(const char *src)
 
 C<String *unquote_special(const char *src)>
 
-Replaces every occurrance in C<src> of C<"\a">, C<"\b">, C<"\f">, C<"\n">,
+Replaces every occurrence in C<src> of C<"\a">, C<"\b">, C<"\f">, C<"\n">,
 C<"\r">, C<"\t"> or C<"\v"> with the corresponding special characters (as
 interpreted by I<C>). Ascii codes in octal or hexadecimal (i.e. "\ooo" or
 "\xhh") are replaced with their corresponding ASCII characters. The caller
@@ -216,7 +232,7 @@ static String *unquote_special(const char *src)
 
 C<String *quote_equals(const char *src)>
 
-Replace every occurrance in C<src> of C<"="> with C<"\=">. The caller must
+Replace every occurrence in C<src> of C<"="> with C<"\=">. The caller must
 deallocate the memory returned.
 
 */
@@ -230,7 +246,7 @@ static String *quote_equals(const char *src)
 
 C<String *quote_equals(const char *src)>
 
-Replace every occurrance in C<src> of C<"\="> with C<"=">. The caller must
+Replace every occurrence in C<src> of C<"\="> with C<"=">. The caller must
 deallocate the memory returned.
 
 */
@@ -270,7 +286,9 @@ C<void prop_parse(Map *map, const char *path, char *line, size_t lineno)>
 Parses a line from a properties file. C<path> if the path of the properties
 file. C<line> is the text to parse. C<lineno> is the current line number.
 The property parsed, if any, is added to C<map>. Emits error messages when
-syntax errors occur.
+syntax errors occur. That's probably a mistake. To suppress the error
+messages, call I<prog_err_none(3)> first and restore error message afterwards
+with something like I<prog_err_stderr(3)>.
 
 */
 
@@ -290,23 +308,37 @@ static void prop_parse(Map *map, const char *path, char *line, size_t lineno)
 	/* Find first unquoted '=' */
 
 	for (p = cstr(prop), eq = strchr(p, '='); eq; eq = strchr(eq + 1, '='))
-		if (eq != p && eq[-1] != '\\')
+		if (eq == p || eq[-1] != '\\')
 			break;
 
 	if (!eq)
 	{
 		error("prop: %s line %d: Expected '='\n%s", path, lineno, line);
 		str_release(prop);
+		set_errno(EINVAL);
 		return;
 	}
 
 	/* Identify and separate the name and value */
 
 	value = eq + 1;
-	while (eq > p && isspace((int)(unsigned char)eq[-1]))
+
+	while (is_space(value[0]))
+		++value;
+
+	while (eq > p && is_space(eq[-1]) && (eq == p + 1 || eq[-2] != '\\'))
 		--eq;
+
 	*eq = nul;
 
+	/* Unquote any quoted trailing space in the key */
+
+	if (eq > p + 1 && is_space(eq[-1]) && eq[-2] == '\\')
+	{
+		eq[-2] = eq[-1];
+		eq[-1] = nul;
+	}
+
 	/* Unquote any quoted '=' in the name */
 
 	if (!(name = unquote_equals(p)))
@@ -316,18 +348,15 @@ static void prop_parse(Map *map, const char *path, char *line, size_t lineno)
 		return;
 	}
 
-	/* Add this property to the map */
-
-	if (str_length(name) == 0)
-	{
-		error("prop: %s line %d: Empty name\n%s", path, lineno, line);
-		str_release(prop);
-		str_release(name);
-		return;
-	}
-
 	key = cstr(name);
 
+	/* Unquote any quoted leading space in the value */
+
+	if (*value == '\\')
+		++value;
+
+	/* Add this property to the map */
+
 	if (!(val = mem_strdup(value)))
 	{
 		error("prop: Out of memory");
@@ -337,7 +366,10 @@ static void prop_parse(Map *map, const char *path, char *line, size_t lineno)
 	}
 
 	if (map_add(map, key, val) == -1)
+	{
 		error("prop: %s line %d: Property %s already defined\n%s", path, lineno, name, line);
+		mem_release(val);
+	}
 
 	str_release(prop);
 	str_release(name);
@@ -348,7 +380,8 @@ static void prop_parse(Map *map, const char *path, char *line, size_t lineno)
 C<Prop *prop_load(const char *path, Prop *defaults)>
 
 Creates and returns a new I<Prop> containing C<defaults> and the properties
-obtained from the properties file named C<path>. On error, returns C<NULL>.
+obtained from the properties file named C<path>. On error, returns C<null>
+with C<errno> set appropriately.
 
 */
 
@@ -379,9 +412,9 @@ static Prop *prop_load(const char *path, Prop *defaults)
 
 C<int prop_init(void)>
 
-Initialises the I<prop> module. Loads properties from the following locations:
+Initialises the L<prop(3)|prop(3)> module. Loads properties from the following locations:
 
-    /etc/properties.app             - system-wide, generic properties
+    /etc/properties/app             - system-wide, generic properties
     $HOME/.properties/app           - user-defined, generic properties
     /etc/properties/app.progname    - system-wide, program-specific properties
     $HOME/.properties/app.progname  - user-defined, program-specific properties
@@ -411,18 +444,15 @@ static int prop_init(void)
 	/* System wide generic properties: /etc/properties/app */
 
 	snprintf(path, path_len, "%s%cproperties%capp", ETC_DIR, PATH_SEP, PATH_SEP);
-	prop_next = prop_load(path, prop);
-	if (prop_next)
+	if ((prop_next = prop_load(path, prop)))
 		prop = prop_next;
 
 	/* User defined generic properties: $HOME/.properties/app */
 
-	home = user_home();
-	if (home)
+	if ((home = user_home()))
 	{
 		snprintf(path, path_len, "%s%c.properties%capp", home, PATH_SEP, PATH_SEP);
-		prop_next = prop_load(path, prop);
-		if (prop_next)
+		if ((prop_next = prop_load(path, prop)))
 			prop = prop_next;
 	}
 
@@ -443,8 +473,7 @@ static int prop_init(void)
 		/* System wide program specific properties: /etc/properties/app.progname */
 
 		snprintf(path, path_len, "%s%cproperties%capp.%s", ETC_DIR, PATH_SEP, PATH_SEP, progname);
-		prop_next = prop_load(path, prop);
-		if (prop_next)
+		if ((prop_next = prop_load(path, prop)))
 			prop = prop_next;
 
 		/* User defined program specific properties: $HOME/.properties/app.progname */
@@ -452,8 +481,7 @@ static int prop_init(void)
 		if (home)
 		{
 			snprintf(path, path_len, "%s%c.properties%capp.%s", home, PATH_SEP, PATH_SEP, progname);
-			prop_next = prop_load(path, prop);
-			if (prop_next)
+			if ((prop_next = prop_load(path, prop)))
 			{
 				prop = prop_next;
 				writable = 1;
@@ -498,8 +526,8 @@ static int prop_init(void)
 
 =item C<const char *prop_get(const char *name)>
 
-Returns the value of the property named C<name>. Returns C<NULL> if there is
-no such property.
+Returns the value of the property named C<name>. Returns C<null> if there is
+no such property. On error, returns C<null> with C<errno> set appropriately.
 
 =cut
 
@@ -509,9 +537,10 @@ const char *prop_get(const char *name)
 {
 	Prop *p;
 	const char *value = NULL;
+	int err;
 
-	if (locker_wrlock(g.locker) == -1)
-		return NULL;
+	if ((err = locker_wrlock(g.locker)))
+		return set_errnull(err);
 
 	if (!g.init && prop_init() == -1)
 	{
@@ -523,8 +552,8 @@ const char *prop_get(const char *name)
 		if ((value = map_get(p->map, name)))
 			break;
 
-	if (locker_unlock(g.locker) == -1)
-		return NULL;
+	if ((err = locker_unlock(g.locker)))
+		return set_errnull(err);
 
 	return value;
 }
@@ -533,8 +562,8 @@ const char *prop_get(const char *name)
 
 =item C<const char *prop_get_or(const char *name, const char *default_value)>
 
-Returns the value of the property named C<name>. Returns C<default_value> if
-there is no such property.
+Returns the value of the property named C<name>. Returns C<default_value> on
+error or if there is no such property.
 
 =cut
 
@@ -551,10 +580,11 @@ const char *prop_get_or(const char *name, const char *default_value)
 
 =item C<const char *prop_set(const char *name, const char *value)>
 
-Sets the property named C<name> to a copy of C<value>. On success, returns
-the copy of C<value>. On error, returns C<NULL>. If I<prop_save()> is called
-after a call to this function, the new property will be saved to disk and
-will be available the next time this program is executed.
+Sets the property named C<name> to a copy of C<value>. If I<prop_save(3)> is
+called after a call to this function, the new property will be saved to disk
+and will be available the next time this program is executed. On success,
+returns the copy of C<value>. On error, returns C<null> with C<errno> set
+appropriately.
 
 =cut
 
@@ -563,9 +593,10 @@ will be available the next time this program is executed.
 const char *prop_set(const char *name, const char *value)
 {
 	char *val;
+	int err;
 
-	if (locker_wrlock(g.locker) == -1)
-		return NULL;
+	if ((err = locker_wrlock(g.locker)))
+		return set_errnull(err);
 
 	if (!g.init && prop_init() == -1)
 	{
@@ -588,8 +619,8 @@ const char *prop_set(const char *name, const char *value)
 
 	g.dirty = 1;
 
-	if (locker_unlock(g.locker) == -1)
-		return NULL;
+	if ((err = locker_unlock(g.locker)))
+		return set_errnull(err);
 
 	return val;
 }
@@ -599,8 +630,8 @@ const char *prop_set(const char *name, const char *value)
 =item C<int prop_get_int(const char *name)>
 
 Returns the value of the property named C<name> as an integer. Returns C<0>
-if there is no such property or if it is not interpretable as a decimal
-integer.
+on error or if there is no such property or if it is not interpretable as a
+decimal integer.
 
 =cut
 
@@ -616,8 +647,8 @@ int prop_get_int(const char *name)
 =item C<int prop_get_int_or(const char *name, int default_value)>
 
 Returns the value of the property named C<name> as an integer. Returns
-C<default_value> if there is no such property or of it is not interpretable
-as a decimal integer.
+C<default_value> on error or if there is no such property or if it is not
+interpretable as a decimal integer.
 
 =cut
 
@@ -635,10 +666,10 @@ int prop_get_int_or(const char *name, int default_value)
 
 =item C<int prop_set_int(const char *name, int value)>
 
-Sets the property named C<name> to C<value>. On success, returns C<value>.
-On error, returns 0. If I<prop_save()> is called after a call to this
-function, the new property will be saved to disk and will be available the
-next time this program is executed.
+Sets the property named C<name> to C<value>. If I<prop_save(3)> is called
+after a call to this function, the new property will be saved to disk and
+will be available the next time this program is executed. On success,
+returns C<value>. On error, returns C<0>.
 
 =cut
 
@@ -655,9 +686,9 @@ int prop_set_int(const char *name, int value)
 
 =item C<double prop_get_double(const char *name)>
 
-Returns the value of the property named C<name> as a double. Returns 0.0
-if there is no such property or if it is not interpretable as a floating
-point number.
+Returns the value of the property named C<name> as a double. Returns C<0.0>
+on error or if there is no such property or if it is not interpretable as a
+floating point number.
 
 =cut
 
@@ -673,8 +704,8 @@ double prop_get_double(const char *name)
 =item C<double prop_get_double_or(const char *name, double default_value)>
 
 Returns the value of the property named C<name> as a double. Returns
-C<default_value> if there is no such property or of it is not interpretable
-as a floating point number.
+C<default_value> on error or if there is no such property or if it is not
+interpretable as a floating point number.
 
 =cut
 
@@ -692,10 +723,10 @@ double prop_get_double_or(const char *name, double default_value)
 
 =item C<double prop_set_double(const char *name, double value)>
 
-Sets the property named C<name> to C<value>. On success, returns C<value>.
-On error, returns 0.0. If I<prop_save()> is called after a call to this
-function, the new property will be saved to disk and will be available the
-next time this program is executed.
+Sets the property named C<name> to C<value>. If I<prop_save(3)> is called
+after a call to this function, the new property will be saved to disk and
+will be available the next time this program is executed. On success,
+returns C<value>. On error, returns C<0.0>.
 
 =cut
 
@@ -713,9 +744,9 @@ double prop_set_double(const char *name, double value)
 =item C<int prop_get_bool(const char *name)>
 
 Returns the value of the property named C<name> as a boolean value. Returns
-C<0> if there is no such property or if it is not interpretable as a boolean
-value. The values: C<"true">, C<"yes">, C<"on"> and C<"1"> are all
-interpreted as C<true>. All other values are C<false>.
+C<0> on error or if there is no such property. The values: C<"true">,
+C<"yes">, C<"on"> and C<"1"> are all interpreted as C<true>. All other
+values are interpreted as C<false>.
 
 =cut
 
@@ -731,11 +762,11 @@ int prop_get_bool(const char *name)
 =item C<int prop_get_bool_or(const char *name, int default_value)>
 
 Returns the value of the property named C<name> as a boolean value. Returns
-C<default_value> if there is no such property or of it is not interpretable
-as a boolean value. The values: C<"true">, C<"yes">, C<"on"> and C<"1"> are
-all interpreted as C<true>. The values: C<"false">, C<"no">, C<"off"> and
-C<"0"> are all interpreted as C<false>. All other values are disregarded and
-will cause C<default_value> to be returned.
+C<default_value> on error or if there is no such property or if it is not
+interpretable as a boolean value. The values: C<"true">, C<"yes">, C<"on">
+and C<"1"> are all interpreted as C<true>. The values: C<"false">, C<"no">,
+C<"off"> and C<"0"> are all interpreted as C<false>. All other values are
+disregarded and will cause C<default_value> to be returned.
 
 =cut
 
@@ -800,10 +831,10 @@ int prop_get_bool_or(const char *name, int default_value)
 
 =item C<int prop_set_bool(const char *name, int value)>
 
-Sets the property named C<name> to C<value>. On success, returns C<value>.
-On error, returns 0. If I<prop_save()> is called after a call to this
-function, the new property will be saved to disk and will be available the
-next time this program is executed.
+Sets the property named C<name> to C<value>. If I<prop_save(3)> is called
+after a call to this function, the new property will be saved to disk and
+will be available the next time this program is executed. On success,
+returns C<value>. On error, returns C<0>.
 
 =cut
 
@@ -819,9 +850,10 @@ int prop_set_bool(const char *name, int value)
 =item C<int prop_unset(const char *name)>
 
 Removes the property named C<name>. Property removal is only saved to disk
-when I<prop_save()> is called, if the property existed only in the
+when I<prop_save(3)> is called, if the property existed only in the
 user-defined, program-specific properties file, or was created by the
-program at runtime.
+program at runtime. On success, returns C<0>. On error, returns C<-1> with
+C<errno> set appropriately.
 
 =cut
 
@@ -830,9 +862,10 @@ program at runtime.
 int prop_unset(const char *name)
 {
 	Prop *p;
+	int err;
 
-	if (locker_wrlock(g.locker) == -1)
-		return -1;
+	if ((err = locker_wrlock(g.locker)))
+		return set_errno(err);
 
 	if (!g.init && prop_init() == -1)
 	{
@@ -845,8 +878,8 @@ int prop_unset(const char *name)
 
 	g.dirty = 1;
 
-	if (locker_unlock(g.locker) == -1)
-		return -1;
+	if ((err = locker_unlock(g.locker)))
+		return set_errno(err);
 
 	return 0;
 }
@@ -864,8 +897,11 @@ save its own properties. They are saved in the following file:
 
     $HOME/.properties/app.progname
 
-The F<.properties> directory is created if necessary. On success, returns 0.
-On error, returns -1.
+Where C<progname> is the name of the program after being translated as
+described at the top of the DESCRIPTION section.
+
+The F<.properties> directory is created if necessary. On success, returns
+C<0>. On error, returns C<-1> with C<errno> set appropriately.
 
 =cut
 
@@ -882,17 +918,18 @@ int prop_save(void)
 	List *keys;
 	Lister *k;
 	FILE *file;
+	int err;
 
 	if (!prop_get_bool_or("save", 1))
 		return 0;
 
-	if (locker_wrlock(g.locker) == -1)
-		return -1;
+	if ((err = locker_wrlock(g.locker)))
+		return set_errno(err);
 
 	if (!g.dirty)
 	{
-		if (locker_unlock(g.locker) == -1)
-			return -1;
+		if ((err = locker_unlock(g.locker)))
+			return set_errno(err);
 
 		return 0;
 	}
@@ -900,13 +937,13 @@ int prop_save(void)
 	if (!prog_name())
 	{
 		locker_unlock(g.locker);
-		return -1;
+		return set_errno(EINVAL);
 	}
 
 	if (!(home = user_home()))
 	{
 		locker_unlock(g.locker);
-		return -1;
+		return set_errno(EINVAL);
 	}
 
 	path_len = limit_path();
@@ -930,7 +967,7 @@ int prop_save(void)
 	{
 		mem_release(path);
 		locker_unlock(g.locker);
-		return -1;
+		return set_errno(EINVAL);
 	}
 
 	if (!(progname = mem_strdup(prog_name())))
@@ -976,7 +1013,7 @@ int prop_save(void)
 		return -1;
 	}
 
-	while (lister_has_next(k))
+	while (lister_has_next(k) == 1)
 	{
 		const char *key = (const char *)lister_next(k);
 		const char *value = map_get(g.prop->map, key);
@@ -1008,6 +1045,21 @@ int prop_save(void)
 		str_release(lhs);
 		lhs = lhs2;
 
+		/* Quote any trailing space in the key */
+
+		if (is_space(cstr(lhs)[str_length(lhs) - 1]))
+		{
+			if (!str_insert(lhs, str_length(lhs) - 1, "\\"))
+			{
+				fclose(file);
+				lister_release(k);
+				list_release(keys);
+				str_release(lhs);
+				locker_unlock(g.locker);
+				return -1;
+			}
+		}
+
 		/* Quote any special chars in the value */
 
 		if (!(rhs = quote_special(value)))
@@ -1020,6 +1072,21 @@ int prop_save(void)
 			return -1;
 		}
 
+		/* Quote any leading space in the value */
+
+		if (is_space(cstr(rhs)[0]))
+		{
+			if (!str_insert(rhs, 0, "\\"))
+			{
+				fclose(file);
+				lister_release(k);
+				list_release(keys);
+				str_release(lhs);
+				locker_unlock(g.locker);
+				return -1;
+			}
+		}
+
 		fprintf(file, "%s=%s\n", cstr(lhs), cstr(rhs));
 		str_release(lhs);
 		str_release(rhs);
@@ -1030,8 +1097,37 @@ int prop_save(void)
 	list_release(keys);
 	g.dirty = 0;
 
-	if (locker_unlock(g.locker) == -1)
-		return -1;
+	if ((err = locker_unlock(g.locker)))
+		return set_errno(err);
+
+	return 0;
+}
+
+/*
+
+=item C<int prop_clear(void)>
+
+Clears the properties as though they were never initialised. On success,
+returns C<0>. On error, returns C<-1> with C<errno> set appropriately.
+
+=cut
+
+*/
+
+int prop_clear(void)
+{
+	int err;
+
+	if ((err = locker_wrlock(g.locker)))
+		return set_errno(err);
+
+	prop_release(g.prop);
+	g.prop = NULL;
+	g.init = 0;
+	g.dirty = 0;
+
+	if ((err = locker_unlock(g.locker)))
+		return set_errno(err);
 
 	return 0;
 }
@@ -1040,9 +1136,10 @@ int prop_save(void)
 
 =item C<int prop_locker(Locker *locker)>
 
-Sets the locking strategy for the prop module to C<locker>. This is only
-needed in multi threaded programs. It must only be called once, from the
-main thread. On success, returns C<0>. On error, returns C<-1>.
+Sets the locking strategy for the L<prop(3)|prop(3)> module to C<locker>.
+This is only needed in multi threaded programs. It must only be called once,
+from the main thread. On success, returns C<0>. On error, returns C<-1> with
+C<errno> set appropriately.
 
 =cut
 
@@ -1062,6 +1159,21 @@ int prop_locker(Locker *locker)
 
 =back
 
+=head1 ERRORS
+
+On error, C<errno> is set either by an underlying function, or as follows:
+
+=over 4
+
+=item EINVAL
+
+When there is no prog_name or home directory or when there is a parse error
+in a properties file or if the F<~/.properties> exists but is not a
+directory. I<prop_locker(3)> sets this when an attempt is made to change the
+locker that has already been set.
+
+=back
+
 =head1 MT-Level
 
 MT-Disciplined
@@ -1075,6 +1187,7 @@ MT-Disciplined
 
 =head1 EXAMPLE
 
+    #include <slack/std.h>
     #include <slack/prog.h>
     #include <slack/prop.h>
     #include <slack/err.h>
@@ -1098,13 +1211,16 @@ MT-Disciplined
         prop_set_double("double", d *= 1.75);
         prop_set_bool("boolean", !b);
         prop_save();
-        return 0;
+
+        return EXIT_SUCCESS;
     }
 
 =head1 BUGS
 
-This only provides coarse grained persistence. If multiple instances
-of the same program are setting properties, the last to exit wins.
+This only provides coarse grained persistence. If multiple instances of the
+same program are setting properties, the last to exit wins. This can be
+ovecome by calling I<prop_save()> after setting any property and
+I<prop_clear()> before getting any property.
 
 =head1 SEE ALSO
 
@@ -1113,14 +1229,27 @@ L<prog(3)|prog(3)>
 
 =head1 AUTHOR
 
-20010215 raf <raf@raf.org>
+20040102 raf <raf@raf.org>
 
 =cut
 
 */
 
+#endif
+
 #ifdef TEST
 
+static const char *user_home()
+{
+	struct passwd *pwent;
+	char *home = NULL;
+
+	if ((pwent = getpwuid(getuid())))
+		home = pwent->pw_dir;
+
+	return (home && strlen(home)) ? home : NULL;
+}
+
 static int props_exist(void)
 {
 	struct stat status[1];
@@ -1128,8 +1257,7 @@ static int props_exist(void)
 	char *path;
 	size_t path_len;
 
-	home = user_home();
-	if (!home)
+	if (!(home = user_home()))
 		return 0;
 
 	path_len = limit_path();
@@ -1158,8 +1286,7 @@ static void clean(int has_props)
 	size_t path_len;
 	size_t len;
 
-	home = user_home();
-	if (!home)
+	if (!(home = user_home()))
 		return;
 
 	path_len = limit_path();
@@ -1199,16 +1326,17 @@ int main(int ac, char **av)
 	data[] =
 	{
 		{ "key", "value" },
-		{ "key with spaces", "value with spaces" },
+		{ " key with spaces ", "value with spaces" },
 		{ "key with = sign", " value with leading space" },
 		{ "key with newline\n and = two = signs", "value with newline\n!" },
 		{ "key with newline,\n = two = signs and an Escape\033!", "value with newline\n and two non printables\001!\002!" },
+		{ "", "" },
 		{ NULL, NULL }
 	};
 
-	const char *key = "key";
-	const char *value = "value";
-	const char *not_key = "not key";
+	const char * const key = "key";
+	const char * const value = "value";
+	const char * const not_key = "not key";
 	int has_props = props_exist();
 	const char *val;
 	int ival, i;
@@ -1217,15 +1345,21 @@ int main(int ac, char **av)
 	int bool_val;
 	int errors = 0;
 
-	printf("Testing: prop\n");
+	if (ac == 2 && !strcmp(av[1], "help"))
+	{
+		printf("usage: %s\n", *av);
+		return EXIT_SUCCESS;
+	}
+
+	printf("Testing: %s\n", "prop");
 	prog_init();
 
 	val = prop_set(key, value);
 	if (strcmp(val, value))
 		++errors, printf("Test1: prop_set(key, value) failed (%s not %s)\n", val, value);
 	val = prop_get(key);
-	if (strcmp(val, value))
-		++errors, printf("Test2: prop_get(key) failed (%s not %s)\n", val, value);
+	if (!val || strcmp(val, value))
+		++errors, printf("Test2: prop_get(key) failed (%s not %s)\n", val ? val : "null", value);
 	val = prop_get_or(key, NULL);
 	if (strcmp(val, value))
 		++errors, printf("Test3: prop_get_or(key, NULL) failed (%s not %s)\n", val, value);
@@ -1246,10 +1380,7 @@ int main(int ac, char **av)
 		++errors, printf("Test8: prop_save() (without progname) failed (%d not -1)\n", ival);
 
 	prog_set_name("prop.test");
-	prop_release(g.prop);
-	g.prop = NULL;
-	g.init = 0;
-	g.dirty = 0;
+	prop_clear();
 
 	val = prop_get(not_key);
 	if (val != NULL)
@@ -1265,85 +1396,76 @@ int main(int ac, char **av)
 		++errors, printf("Test12: prop_save() (with progname) failed (%d not 0)\n", ival);
 
 	clean(has_props);
-	prop_release(g.prop);
-	g.prop = NULL;
-	g.init = 0;
-	g.dirty = 0;
+	prop_clear();
 
 	for (i = 0; data[i].key; ++i)
 	{
 		val = prop_set(data[i].key, data[i].value);
 		if (strcmp(val, data[i].value))
-			++errors, printf("Test%d: prop_set('%s', '%s') failed ('%s' not '%s')\n", 13 + 2 * i, data[i].key, data[i].value, val, value);
+			++errors, printf("Test%d: prop_set('%s', '%s') failed ('%s' not '%s')\n", 13 + 2 * i, data[i].key, data[i].value, val, data[i].value);
 		val = prop_get(data[i].key);
-		if (strcmp(val, data[i].value))
-			++errors, printf("Test%d: prop_get('%s') failed ('%s' not '%s')\n", 14 + 2 * i, data[i].key, val, value);
+		if (!val || strcmp(val, data[i].value))
+			++errors, printf("Test%d: prop_get('%s') failed ('%s' not '%s')\n", 14 + 2 * i, data[i].key, val ? val : "null", data[i].value);
 	}
 
 	ival = prop_save();
 	if (ival != 0)
-		++errors, printf("Test21: prop_save() (with progname) failed (%d not 0)\n", ival);
+		++errors, printf("Test25: prop_save() (with progname) failed (%d not 0)\n", ival);
 
-	prop_release(g.prop);
-	g.prop = NULL;
-	g.init = 0;
-	g.dirty = 0;
+	prop_clear();
 
 	for (i = 0; data[i].key; ++i)
 	{
 		val = prop_get(data[i].key);
-		if (strcmp(val, data[i].value))
-			++errors, printf("Test%d: prop_get('%s') failed ('%s' not '%s')\n", 22 + i, data[i].key, val, data[i].value);
+		if (!val || strcmp(val, data[i].value))
+			++errors, printf("Test%d: prop_get('%s') failed ('%s' not '%s')\n", 26 + i, data[i].key, val ? val : "null", data[i].value);
 	}
 
 	clean(has_props);
-	prop_release(g.prop);
-	g.prop = NULL;
-	g.init = 0;
-	g.dirty = 0;
+	prop_clear();
 
 	if ((int_val = prop_set_int("i", 37)) != 37)
-		++errors, printf("Test26: prop_set_int() failed (%d not 37)\n", int_val);
+		++errors, printf("Test32: prop_set_int() failed (%d not 37)\n", int_val);
 
 	if ((int_val = prop_get_int("i")) != 37)
-		++errors, printf("Test27: prop_get_int() failed (%d not 37)\n", int_val);
+		++errors, printf("Test33: prop_get_int() failed (%d not 37)\n", int_val);
 
 	if ((int_val = prop_get_int_or("i", 13)) != 37)
-		++errors, printf("Test28: prop_get_int_or() failed (%d not 37)\n", int_val);
+		++errors, printf("Test34: prop_get_int_or() failed (%d not 37)\n", int_val);
 
 	if ((int_val = prop_get_int_or("j", 13)) != 13)
-		++errors, printf("Test29: prop_get_int_or() failed (%d not 13)\n", int_val);
+		++errors, printf("Test35: prop_get_int_or() failed (%d not 13)\n", int_val);
 
 	if ((double_val = prop_set_double("d", 37.0)) != 37.0)
-		++errors, printf("Test30: prop_set_double() failed (%g not 37.0)\n", double_val);
+		++errors, printf("Test36: prop_set_double() failed (%g not 37.0)\n", double_val);
 
 	if ((double_val = prop_get_double("d")) != 37)
-		++errors, printf("Test31: prop_get_double() failed (%g not 37.0)\n", double_val);
+		++errors, printf("Test37: prop_get_double() failed (%g not 37.0)\n", double_val);
 
 	if ((double_val = prop_get_double_or("d", 13.0)) != 37)
-		++errors, printf("Test32: prop_get_double_or() failed (%g not 37.0)\n", double_val);
+		++errors, printf("Test38: prop_get_double_or() failed (%g not 37.0)\n", double_val);
 
 	if ((double_val = prop_get_double_or("e", 13.0)) != 13.0)
-		++errors, printf("Test33: prop_get_double_or() failed (%g not 13.0)\n", double_val);
+		++errors, printf("Test39: prop_get_double_or() failed (%g not 13.0)\n", double_val);
 
 	if ((bool_val = prop_set_bool("b", 1)) != 1)
-		++errors, printf("Test34: prop_set_bool() failed (%d not 1)\n", bool_val);
+		++errors, printf("Test40: prop_set_bool() failed (%d not 1)\n", bool_val);
 
 	if ((bool_val = prop_get_bool("b")) != 1)
-		++errors, printf("Test35: prop_get_bool() failed (%d not 1)\n", bool_val);
+		++errors, printf("Test41: prop_get_bool() failed (%d not 1)\n", bool_val);
 
 	if ((bool_val = prop_get_bool_or("b", 0)) != 1)
-		++errors, printf("Test36: prop_get_bool_or() failed (%d not 1)\n", bool_val);
+		++errors, printf("Test42: prop_get_bool_or() failed (%d not 1)\n", bool_val);
 
 	if ((bool_val = prop_get_bool_or("c", 1)) != 1)
-		++errors, printf("Test37: prop_get_bool_or() failed (%d not 1)\n", bool_val);
+		++errors, printf("Test43: prop_get_bool_or() failed (%d not 1)\n", bool_val);
 
 	if (errors)
-		printf("%d/37 tests failed\n", errors);
+		printf("%d/43 tests failed\n", errors);
 	else
 		printf("All tests passed\n");
 
-	return 0;
+	return (errors == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 #endif
