@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002 raf <raf@raf.org>
+* Copyright (C) 1999-2004 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/copyleft/gpl.html
 *
-* 20020916 raf <raf@raf.org>
+* 20040102 raf <raf@raf.org>
 */
 
 #ifndef LIBSLACK_SIG_H
@@ -31,12 +31,12 @@
 typedef void signal_handler_t(int signo);
 
 _begin_decls
-int signal_set_handler _args ((int signo, int flags, signal_handler_t *handler));
-int signal_addset _args ((int signo_handled, int signo_blocked));
-int signal_received _args ((int signo));
-int signal_raise _args ((int signo));
-int signal_handle _args ((int signo));
-void signal_handle_all _args ((void));
+int signal_set_handler(int signo, int flags, signal_handler_t *handler);
+int signal_addset(int signo_handled, int signo_blocked);
+int signal_received(int signo);
+int signal_raise(int signo);
+int signal_handle(int signo);
+void signal_handle_all(void);
 _end_decls
 
 #endif
